const DATA_URL = new URL("/brochures/", document.baseURI)
const PARSER = new DOMParser()
const SERIALIZER = new XMLSerializer()
const DATE_FORMATTER = new Intl.DateTimeFormat("fr", {dateStyle: "short"})

export async function uploadDocument(documentFile, documentDetails = {}){

    let uniqueName = await getUniqueName(documentDetails)

    let baseURI = new URL(`./${uniqueName}/`, DATA_URL)

    let originalFileName = `${slugify(documentDetails.title)}-original.pdf`

    let uploadProgress = fetch(new URL(`./${originalFileName}`, baseURI), {
            method: "PUT",
            headers: {
                "Content-Type": "application/pdf"
            },
            body: documentFile
        })

    let page = PARSER.parseFromString(await (await fetch("/brochure.template.html")).text(), "text/html")

    page.title = documentDetails.title + " " + page.title

    page.querySelector('[property="name"]').textContent = documentDetails.title
    page.querySelector('a[href="original.pdf"]').href = "./"+originalFileName

    {
        let el = page.querySelector('[property="uploadDate"]')
        el.textContent = DATE_FORMATTER.format(new Date(documentDetails.uploadDate))
        el.setAttribute("datetime", documentDetails.uploadDate);
        el.setAttribute("content", documentDetails.uploadDate);
    }

    if(documentDetails.author){
        page.querySelector('[property="author"]').textContent = documentDetails.author
    } else {
        page.querySelector('[property="author"]').remove()
    }

    let responses = await Promise.all([
        fetch(new URL("./index.html", baseURI), {
            method: "PUT",
            headers: {
                "Content-Type": "text/html"
            },
            body: SERIALIZER.serializeToString(page)
        }),
        uploadProgress
    ]);

    for(let res of responses) {
        if(!res.ok){
            throw new Error(`Network error ${res.status} ${res.statusText}`)
        }
    }

    return baseURI.toString()
}

async function getUniqueName(documentDetails){
    let nameCandidate, isUnique;

    do {
        nameCandidate = getRandomName(documentDetails)

        let res = await fetch(new URL(`./${nameCandidate}/index.html`, DATA_URL), {
            method: "HEAD"
        });

        if(!res.ok && res.status != 404){
            throw new Error(`Network error ${res.status} ${res.statusText}`)
        }

        isUnique = res.status == 404

    } while(!isUnique)


    return nameCandidate
}

function getRandomName(documentDetails){
    let source = new Uint8Array(16)
    crypto.getRandomValues(source)
    let parts = [toHex(source)]

    if(documentDetails.author){
        parts.push(slugify(documentDetails.author))
    }

    if(documentDetails.title){
        parts.push(slugify(documentDetails.title))
    }

    return parts.join("-")
}

function toHex(buffer) {
    return Array.prototype.map.call(buffer, x => x.toString(16).padStart(2, '0')).join('');
}

function slugify(str){
    return str.toLowerCase().replace(/\//g,"-").replace(/\s/g, "-").replace(/--+/, "-")
}